"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.wrapCoercionsForListType = wrapCoercionsForListType;

var _lodash = _interopRequireDefault(require("lodash"));

var _fp = _interopRequireDefault(require("lodash/fp"));

var returnTypes = _interopRequireWildcard(require("../return-types"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Takes an existing mapping of coercions (e.g. user.js COERCIONS constant) and returns
// a mapping where the keys are corresponding to the list of that singular type (e.g. text -> text-list),
// where the coercion functions for the singular type are wrapped in a map, such that any list of singular
// typed values will be each be converted using the underlying singular type coercion function.
//
// For example:
//
// If your coercion map looked like this:
//
// export const COERCIONS = {
//   [TEXT]: _.identity,
//   [NUMBER]: value => +value
// }
//
// And you called wrapCoercionsForListType(COERCIONS), you would get the following:
//
// export const COERCIONS_FOR_LIST_TYPE = {
//   [TEXT_LIST]: fp.map(COERCIONS[TEXT]),
//   [NUMBER_LIST]: fp.map(COERCIONS[NUMBER])
// }
//
// The key here is we never want to do something "special" for converting singular values that are wrapped
// in the list type, it should always delegate to the singular type coercion function for each value
// in the list.
//
// Note: If you attempt to wrap a singular base type which converts to a data type that does _not_ have
// a list counterpart data type, this function will throw an error.
//
function wrapCoercionsForListType(singularTypeCoercions) {
  return _lodash.default.reduce(singularTypeCoercions, function (accumulator, coercionFn, rightSideType) {
    var listDataTypeKey = _lodash.default.toUpper("".concat(rightSideType, "_LIST"));

    var listDataType = _lodash.default.get(returnTypes, listDataTypeKey) || throwWrapCoercionError(listDataTypeKey, rightSideType);
    return _lodash.default.set(accumulator, listDataType, _fp.default.map(coercionFn));
  }, {});
}

function throwWrapCoercionError(listDataTypeKey, rightSideType) {
  var message = "List data type ".concat(listDataTypeKey, " is not supported for singular base type ").concat(rightSideType, " (see ./return-types.js)");
  throw new Error(message);
}
//# sourceMappingURL=util.js.map
